<h3>CFDI de {{ c.TipoDeComprobante | desc }}</h3>
<table>
	<tr>
		<th style="width: 10%">Serie</th>
		<th style="width: 10%">Folio</th>
		<th style="width: 40%">Emisor</th>
		<th style="width: 40%">Receptor</th>
	</tr>
	<tr>
		<td class="cen">{{ c.Serie }}</td>
		<td class="cen">{{ c.Folio }}</td>
		<td rowspan="13">
			<div class="h cen"><b>{{ c.Emisor.Nombre }}</b></div>
			{{ c.Emisor | simple('Rfc', 'RegimenFiscal')}}
			<div><b>C.P.: </b>{{ c.LugarExpedicion | code }}</div>
			<div><b>No. Certificado: </b>{{ c.NoCertificado }}</div>
		</td>
		<td rowspan="13">
			<div class="h cen"><b>{{ c.Receptor.Nombre }}</b></div>
			{{ c.Receptor | simple('Rfc', 'RegimenFiscalReceptor', 'DomicilioFiscalReceptor', 'NumRegIdTrib')}}
		</td>
	</tr>
	<tr>
	  	<th colspan="2">Fecha de Emisión</th>
	</tr>
	<tr>
		<td colspan="2" class="cen">{{ c.Fecha }}</td>
	</tr>
</table>
<table style="margin-top: 2px">
	<tr>
		<th style="width: 20%">Moneda</th>
		<th style="width: 26.67%">Método de Pago</th>
		<th style="width: 26.67%">Forma de Pago</th>
		<th style="width: 26.67%">Uso CFDI</th>
	</tr>
	<tr class="cen">
		<td>{% if c.Moneda != 'XXX' %}{{ c.Moneda }}{% else %}{{ 'XXX' }}{% endif %}</td>
		<td>{{ c.MetodoPago | desc }}</td>
		<td>{{ c.FormaPago | desc }}</td>
		<td>{{ c.Receptor.UsoCFDI | desc }}</td>
	</tr>
	<tr>
		<th>Tipo de Cambio</th>
		<th>Exportación</th>
		<th colspan="2">Condiciones de Pago</th>
	</tr>
	<tr class="cen">
		<td>{{ c.TipoCambio }}</td>
		<td>{{ c.Exportacion | desc }}</td>
		<td colspan="2">{{ c.CondicionesDePago }}</td>
	</tr>
</table>
<br>
{% include 'Conceptos.html' %}
{% if c.CfdiRelacionados %}
	{% with c = c.CfdiRelacionados %}
	{% include 'CfdiRelacionados.html' %}
	{% endwith %}
{% endif %}
{% if c.Addenda %}
<div><br></div>
{{ complements(c, c.Addenda) }}
{% endif %}
<div><br></div>
{{ complements(c, c.Complemento) }}
